@@ -528,6 +528,7 @@ communicatorSetWrapper(CommunicatorObject* self, PyObject* args)
 
     assert(self->wrapper == NULL);
     self->wrapper = wrapper;
+    Py_INCREF(self->wrapper);
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -667,6 +668,39 @@ communicatorFindObjectFactory(CommunicatorObject* self, PyObject* args)
     return pof->find(id);
 }
 
+#ifdef WIN32
+extern "C"
+#endif
+static PyObject*
+communicatorSetDefaultContext(CommunicatorObject* self, PyObject* args)
+{
+    PyObject* dict;
+    if(!PyArg_ParseTuple(args, STRCAST("O!"), &PyDict_Type, &dict))
+    {
+        return NULL;
+    }
+
+    Ice::Context ctx;
+    if(!dictionaryToContext(dict, ctx))
+    {
+	return NULL;
+    }
+
+    try
+    {
+        (*self->communicator)->setDefaultContext(ctx);
+    }
+    catch(const Ice::Exception& ex)
+    {
+        setPythonException(ex);
+        return NULL;
+
+    }
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 #ifdef WIN32
 extern "C"
 #endif
@@ -998,6 +1032,8 @@ static PyMethodDef CommunicatorMethods[] =
         PyDoc_STR(STRCAST("addObjectFactory(factory, id) -> None")) },
     { STRCAST("findObjectFactory"), (PyCFunction)communicatorFindObjectFactory, METH_VARARGS,
         PyDoc_STR(STRCAST("findObjectFactory(id) -> Ice.ObjectFactory")) },
+    { STRCAST("setDefaultContext"), (PyCFunction)communicatorSetDefaultContext, METH_VARARGS,
+        PyDoc_STR(STRCAST("setDefaultContext(ctx) -> None")) },
     { STRCAST("getDefaultContext"), (PyCFunction)communicatorGetDefaultContext, METH_NOARGS,
         PyDoc_STR(STRCAST("getDefaultContext() -> Ice.Context")) },
     { STRCAST("getProperties"), (PyCFunction)communicatorGetProperties, METH_NOARGS,
