@@ -618,9 +618,9 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
     {
     case PrimitiveInfo::KindBool:
     {
-        int sz = PySequence_Fast_GET_SIZE(fs.get());
+        Py_ssize_t sz = PySequence_Fast_GET_SIZE(fs.get());
         Ice::BoolSeq seq(sz);
-        for(int i = 0; i < sz; ++i)
+        for(Py_ssize_t i = 0; i < sz; ++i)
         {
             PyObject* item = PySequence_Fast_GET_ITEM(fs.get(), i);
             if(item == NULL)
@@ -630,7 +630,8 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
             int isTrue = PyObject_IsTrue(item);
             if(isTrue < 0)
             {
-                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<bool>"), i);
+                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<bool>"),
+			     static_cast<int>(i));
                 throw AbortMarshaling();
             }
             seq[i] = isTrue ? true : false;
@@ -640,9 +641,9 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
     }
     case PrimitiveInfo::KindByte:
     {
-        int sz = PySequence_Fast_GET_SIZE(fs.get());
+        Py_ssize_t sz = PySequence_Fast_GET_SIZE(fs.get());
         Ice::ByteSeq seq(sz);
-        for(int i = 0; i < sz; ++i)
+        for(Py_ssize_t i = 0; i < sz; ++i)
         {
             PyObject* item = PySequence_Fast_GET_ITEM(fs.get(), i);
             if(item == NULL)
@@ -662,7 +663,8 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
 
             if(PyErr_Occurred() || val < 0 || val > 255)
             {
-                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<byte>"), i);
+                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<byte>"),
+			     static_cast<int>(i));
                 throw AbortMarshaling();
             }
             seq[i] = static_cast<Ice::Byte>(val);
@@ -672,9 +674,9 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
     }
     case PrimitiveInfo::KindShort:
     {
-        int sz = PySequence_Fast_GET_SIZE(fs.get());
+        Py_ssize_t sz = PySequence_Fast_GET_SIZE(fs.get());
         Ice::ShortSeq seq(sz);
-        for(int i = 0; i < sz; ++i)
+        for(Py_ssize_t i = 0; i < sz; ++i)
         {
             PyObject* item = PySequence_Fast_GET_ITEM(fs.get(), i);
             if(item == NULL)
@@ -694,7 +696,8 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
 
             if(PyErr_Occurred() || val < SHRT_MIN || val > SHRT_MAX)
             {
-                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<short>"), i);
+                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<short>"),
+			     static_cast<int>(i));
                 throw AbortMarshaling();
             }
             seq[i] = static_cast<Ice::Short>(val);
@@ -704,9 +707,9 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
     }
     case PrimitiveInfo::KindInt:
     {
-        int sz = PySequence_Fast_GET_SIZE(fs.get());
+        Py_ssize_t sz = PySequence_Fast_GET_SIZE(fs.get());
         Ice::IntSeq seq(sz);
-        for(int i = 0; i < sz; ++i)
+        for(Py_ssize_t i = 0; i < sz; ++i)
         {
             PyObject* item = PySequence_Fast_GET_ITEM(fs.get(), i);
             if(item == NULL)
@@ -725,13 +728,15 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
             }
             else
             {
-                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<int>"), i);
+                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<int>"),
+			     static_cast<int>(i));
                 throw AbortMarshaling();
             }
 
             if(PyErr_Occurred() || val < INT_MIN || val > INT_MAX)
             {
-                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<int>"), i);
+                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<int>"),
+			     static_cast<int>(i));
                 throw AbortMarshaling();
             }
             seq[i] = static_cast<Ice::Int>(val);
@@ -741,9 +746,9 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
     }
     case PrimitiveInfo::KindLong:
     {
-        int sz = PySequence_Fast_GET_SIZE(fs.get());
+        Py_ssize_t sz = PySequence_Fast_GET_SIZE(fs.get());
         Ice::LongSeq seq(sz);
-        for(int i = 0; i < sz; ++i)
+        for(Py_ssize_t i = 0; i < sz; ++i)
         {
             PyObject* item = PySequence_Fast_GET_ITEM(fs.get(), i);
             if(item == NULL)
@@ -762,13 +767,15 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
             }
             else
             {
-                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<int>"), i);
+                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<int>"),
+			     static_cast<int>(i));
                 throw AbortMarshaling();
             }
 
             if(PyErr_Occurred())
             {
-                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<int>"), i);
+                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<int>"),
+			     static_cast<int>(i));
                 throw AbortMarshaling();
             }
             seq[i] = val;
@@ -778,9 +785,9 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
     }
     case PrimitiveInfo::KindFloat:
     {
-        int sz = PySequence_Fast_GET_SIZE(fs.get());
+        Py_ssize_t sz = PySequence_Fast_GET_SIZE(fs.get());
         Ice::FloatSeq seq(sz);
-        for(int i = 0; i < sz; ++i)
+        for(Py_ssize_t i = 0; i < sz; ++i)
         {
             PyObject* item = PySequence_Fast_GET_ITEM(fs.get(), i);
             if(item == NULL)
@@ -795,7 +802,8 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
             }
             else
             {
-                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<float>"), i);
+                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<float>"),
+			     static_cast<int>(i));
                 throw AbortMarshaling();
             }
 
@@ -806,9 +814,9 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
     }
     case PrimitiveInfo::KindDouble:
     {
-        int sz = PySequence_Fast_GET_SIZE(fs.get());
+        Py_ssize_t sz = PySequence_Fast_GET_SIZE(fs.get());
         Ice::DoubleSeq seq(sz);
-        for(int i = 0; i < sz; ++i)
+        for(Py_ssize_t i = 0; i < sz; ++i)
         {
             PyObject* item = PySequence_Fast_GET_ITEM(fs.get(), i);
             if(item == NULL)
@@ -823,7 +831,8 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
             }
             else
             {
-                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<double>"), i);
+                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<double>"),
+			     static_cast<int>(i));
                 throw AbortMarshaling();
             }
 
@@ -834,9 +843,9 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
     }
     case PrimitiveInfo::KindString:
     {
-        int sz = PySequence_Fast_GET_SIZE(fs.get());
+        Py_ssize_t sz = PySequence_Fast_GET_SIZE(fs.get());
         Ice::StringSeq seq(sz);
-        for(int i = 0; i < sz; ++i)
+        for(Py_ssize_t i = 0; i < sz; ++i)
         {
             PyObject* item = PySequence_Fast_GET_ITEM(fs.get(), i);
             if(item == NULL)
@@ -851,7 +860,8 @@ IcePy::PrimitiveInfo::marshalSequence(PyObject* p, const Ice::OutputStreamPtr& o
             }
             else if(p != Py_None)
             {
-                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<string>"), i);
+                PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of sequence<string>"),
+			     static_cast<int>(i));
                 throw AbortMarshaling();
             }
 
@@ -1188,8 +1198,7 @@ IcePy::StructInfo::marshal(PyObject* p, const Ice::OutputStreamPtr& os, ObjectMa
 {
     assert(PyObject_IsInstance(p, pythonType.get()) == 1); // validate() should have caught this.
 
-    int i = 0;
-    for(DataMemberList::iterator q = members.begin(); q != members.end(); ++q, ++i)
+    for(DataMemberList::iterator q = members.begin(); q != members.end(); ++q)
     {
         DataMemberPtr member = *q;
         char* memberName = const_cast<char*>(member->name.c_str());
@@ -1223,8 +1232,7 @@ IcePy::StructInfo::unmarshal(const Ice::InputStreamPtr& is, const UnmarshalCallb
         throw AbortMarshaling();
     }
 
-    int i = 0;
-    for(DataMemberList::iterator q = members.begin(); q != members.end(); ++q, ++i)
+    for(DataMemberList::iterator q = members.begin(); q != members.end(); ++q)
     {
         DataMemberPtr member = *q;
         member->type->unmarshal(is, member, p.get(), 0);
@@ -1313,9 +1321,9 @@ IcePy::SequenceInfo::marshal(PyObject* p, const Ice::OutputStreamPtr& os, Object
         return;
     }
 
-    int sz = PySequence_Fast_GET_SIZE(fastSeq.get());
+    Py_ssize_t sz = PySequence_Fast_GET_SIZE(fastSeq.get());
     os->writeSize(sz);
-    for(int i = 0; i < sz; ++i)
+    for(Py_ssize_t i = 0; i < sz; ++i)
     {
         PyObject* item = PySequence_Fast_GET_ITEM(fastSeq.get(), i);
         if(item == NULL)
@@ -1324,7 +1332,7 @@ IcePy::SequenceInfo::marshal(PyObject* p, const Ice::OutputStreamPtr& os, Object
         }
         if(!elementType->validate(item))
         {
-            PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of `%s'"), i,
+            PyErr_Format(PyExc_ValueError, STRCAST("invalid value for element %d of `%s'"), static_cast<int>(i),
 			 const_cast<char*>(id.c_str()));
             throw AbortMarshaling();
         }
@@ -1388,17 +1396,17 @@ IcePy::SequenceInfo::print(PyObject* value, IceUtil::Output& out, PrintObjectHis
 	    return;
 	}
 
-	int sz = PySequence_Fast_GET_SIZE(fastSeq.get());
+	Py_ssize_t sz = PySequence_Fast_GET_SIZE(fastSeq.get());
 
 	out.sb();
-	for(int i = 0; i < sz; ++i)
+	for(Py_ssize_t i = 0; i < sz; ++i)
 	{
 	    PyObject* item = PySequence_Fast_GET_ITEM(fastSeq.get(), i);
 	    if(item == NULL)
 	    {
 		break;
 	    }
-	    out << nl << '[' << i << "] = ";
+	    out << nl << '[' << static_cast<int>(i) << "] = ";
 	    elementType->print(item, out, history);
 	}
 	out.eb();
@@ -1450,10 +1458,10 @@ IcePy::DictionaryInfo::marshal(PyObject* p, const Ice::OutputStreamPtr& os, Obje
         throw AbortMarshaling();
     }
 
-    int sz = PyDict_Size(p);
+    Py_ssize_t sz = PyDict_Size(p);
     os->writeSize(sz);
 
-    int pos = 0;
+    Py_ssize_t pos = 0;
     PyObject* key;
     PyObject* value;
     while(PyDict_Next(p, &pos, &key, &value))
@@ -1543,7 +1551,7 @@ IcePy::DictionaryInfo::print(PyObject* value, IceUtil::Output& out, PrintObjectH
     }
     else
     {
-	int pos = 0;
+	Py_ssize_t pos = 0;
 	PyObject* elemKey;
 	PyObject* elemValue;
 	out.sb();
@@ -2062,7 +2070,7 @@ IcePy::ReadObjectCallback::invoke(const Ice::ObjectPtr& p)
 void
 IcePy::ExceptionInfo::marshal(PyObject* p, const Ice::OutputStreamPtr& os, ObjectMap* objectMap)
 {
-    if(!PyInstance_Check(p) || !PyObject_IsInstance(p, pythonType.get()))
+    if(!PyObject_IsInstance(p, pythonType.get()))
     {
         PyErr_Format(PyExc_ValueError, STRCAST("expected exception %s"), id.c_str());
         throw AbortMarshaling();
@@ -2137,7 +2145,7 @@ IcePy::ExceptionInfo::unmarshal(const Ice::InputStreamPtr& is)
 void
 IcePy::ExceptionInfo::print(PyObject* value, IceUtil::Output& out)
 {
-    if(!PyInstance_Check(value) || !PyObject_IsInstance(value, pythonType.get()))
+    if(!PyObject_IsInstance(value, pythonType.get()))
     {
 	out << "<invalid value - expected " << id << ">";
 	return;
@@ -2460,8 +2468,8 @@ IcePy_defineEnum(PyObject*, PyObject* args)
     info->pythonType = type;
     Py_INCREF(type);
 
-    int sz = PyTuple_GET_SIZE(enumerators);
-    for(int i = 0; i < sz; ++i)
+    Py_ssize_t sz = PyTuple_GET_SIZE(enumerators);
+    for(Py_ssize_t i = 0; i < sz; ++i)
     {
         PyObjectHandle e = PyTuple_GET_ITEM(enumerators, i);
         Py_INCREF(e.get());
@@ -2492,8 +2500,8 @@ IcePy_defineStruct(PyObject*, PyObject* args)
     info->pythonType = type;
     Py_INCREF(type);
 
-    int sz = PyTuple_GET_SIZE(members);
-    for(int i = 0; i < sz; ++i)
+    Py_ssize_t sz = PyTuple_GET_SIZE(members);
+    for(Py_ssize_t i = 0; i < sz; ++i)
     {
         PyObject* m = PyTuple_GET_ITEM(members, i);
         assert(PyTuple_Check(m));
@@ -2671,7 +2679,7 @@ IcePy_defineClass(PyObject*, PyObject* args)
         assert(info->base);
     }
 
-    int i, sz;
+    Py_ssize_t i, sz;
     sz = PyTuple_GET_SIZE(interfaces);
     for(i = 0; i < sz; ++i)
     {
@@ -2718,7 +2726,11 @@ IcePy_defineException(PyObject*, PyObject* args)
         return NULL;
     }
 
+#ifdef ICEPY_OLD_EXCEPTIONS
     assert(PyClass_Check(type));
+#else
+    assert(PyExceptionClass_Check(type));
+#endif
     assert(PyTuple_Check(members));
 
     ExceptionInfoPtr info = new ExceptionInfo;
@@ -2732,8 +2744,8 @@ IcePy_defineException(PyObject*, PyObject* args)
 
     info->usesClasses = false;
 
-    int sz = PyTuple_GET_SIZE(members);
-    for(int i = 0; i < sz; ++i)
+    Py_ssize_t sz = PyTuple_GET_SIZE(members);
+    for(Py_ssize_t i = 0; i < sz; ++i)
     {
         PyObject* m = PyTuple_GET_ITEM(members, i);
         assert(PyTuple_Check(m));
